/*
 * ENCRIPT.C - Calcula checkcode, faz a encriptacao de um pacote
 */

 #include <stdlib.h>

 #include <icqtlkt.h>
 #include "icqv5.h"

/*---[ Implementacao ]---------------------------------------------------------------------------*/

 static const unsigned char table[] = {
  0x59, 0x60, 0x37, 0x6B, 0x65, 0x62, 0x46, 0x48,
  0x53, 0x61, 0x4C, 0x59, 0x60, 0x57, 0x5B, 0x3D,
  0x5E, 0x34, 0x6D, 0x36, 0x50, 0x3F, 0x6F, 0x67,
  0x53, 0x61, 0x4C, 0x59, 0x40, 0x47, 0x63, 0x39,
  0x50, 0x5F, 0x5F, 0x3F, 0x6F, 0x47, 0x43, 0x69,
  0x48, 0x33, 0x31, 0x64, 0x35, 0x5A, 0x4A, 0x42,
  0x56, 0x40, 0x67, 0x53, 0x41, 0x07, 0x6C, 0x49,
  0x58, 0x3B, 0x4D, 0x46, 0x68, 0x43, 0x69, 0x48,
  0x33, 0x31, 0x44, 0x65, 0x62, 0x46, 0x48, 0x53,
  0x41, 0x07, 0x6C, 0x69, 0x48, 0x33, 0x51, 0x54,
  0x5D, 0x4E, 0x6C, 0x49, 0x38, 0x4B, 0x55, 0x4A,
  0x62, 0x46, 0x48, 0x33, 0x51, 0x34, 0x6D, 0x36,
  0x50, 0x5F, 0x5F, 0x5F, 0x3F, 0x6F, 0x47, 0x63,
  0x59, 0x40, 0x67, 0x33, 0x31, 0x64, 0x35, 0x5A,
  0x6A, 0x52, 0x6E, 0x3C, 0x51, 0x34, 0x6D, 0x36,
  0x50, 0x5F, 0x5F, 0x3F, 0x4F, 0x37, 0x4B, 0x35,
  0x5A, 0x4A, 0x62, 0x66, 0x58, 0x3B, 0x4D, 0x66,
  0x58, 0x5B, 0x5D, 0x4E, 0x6C, 0x49, 0x58, 0x3B,
  0x4D, 0x66, 0x58, 0x3B, 0x4D, 0x46, 0x48, 0x53,
  0x61, 0x4C, 0x59, 0x40, 0x67, 0x33, 0x31, 0x64,
  0x55, 0x6A, 0x32, 0x3E, 0x44, 0x45, 0x52, 0x6E,
  0x3C, 0x31, 0x64, 0x55, 0x6A, 0x52, 0x4E, 0x6C,
  0x69, 0x48, 0x53, 0x61, 0x4C, 0x39, 0x30, 0x6F,
  0x47, 0x63, 0x59, 0x60, 0x57, 0x5B, 0x3D, 0x3E,
  0x64, 0x35, 0x3A, 0x3A, 0x5A, 0x6A, 0x52, 0x4E,
  0x6C, 0x69, 0x48, 0x53, 0x61, 0x6C, 0x49, 0x58,
  0x3B, 0x4D, 0x46, 0x68, 0x63, 0x39, 0x50, 0x5F,
  0x5F, 0x3F, 0x6F, 0x67, 0x53, 0x41, 0x25, 0x41,
  0x3C, 0x51, 0x54, 0x3D, 0x5E, 0x54, 0x5D, 0x4E,
  0x4C, 0x39, 0x50, 0x5F, 0x5F, 0x5F, 0x3F, 0x6F,
  0x47, 0x43, 0x69, 0x48, 0x33, 0x51, 0x54, 0x5D,
  0x6E, 0x3C, 0x31, 0x64, 0x35, 0x5A, 0x00, 0x00,
 };

/*---[ Implementacao ]---------------------------------------------------------------------------*/

 unsigned long checkcode(unsigned char *packet, short PL)
 {
    unsigned long number1;
    int           r1, r2;
    unsigned long number2;

    number1 = packet[8];
    number1 <<= 8;
    number1 |= packet[4];
    number1 <<= 8;
    number1 |= packet[2];
    number1 <<= 8;
    number1 |= packet[6];

#ifdef NO_RANDOM_NUMBERS
    r1 = 0x2319 % (PL - 0x18);
    r2 = 0x67;
#else
    r1 = 0x18 + rand() % (PL - 0x18);
    r2 = rand() % 0xFF;
#endif

    number2 = r1;
    number2 <<= 8;
    number2 |= packet[r1];
    number2 <<= 8;
    number2 |= r2;
    number2 <<= 8;
    number2 |= table[r2];
    number2 ^= 0xff00ff;

    return number1 ^ number2;
 }

 void encript(unsigned char *pktptr, int pl, unsigned long cc)
 {
     int                 pos;
     unsigned long       code = pl * 0x68656c6c + cc;
     unsigned long       a[6];
     unsigned long       *ptr;

     for(pos=0xA;pos < (pl + 3);pos+=4)
     {
        ptr   = (ULONG *) (pktptr+pos);
        *ptr ^= (code + table[pos & 0xFF]);
     }

     /*

      Scrambling the checkcode.
      Assuming that cc holds the checkcode.

      */

      a[1] = cc & 0x0000001F;
      a[2] = cc & 0x03E003E0;
      a[3] = cc & 0xF8000400;
      a[4] = cc & 0x0000F800;
      a[5] = cc & 0x041F0000;

      a[1] <<= 0x0C;
      a[2] <<= 0x01;
      a[3] >>= 0x0A;
      a[4] <<= 0x10;
      a[5] >>= 0x0F;

     ((OUTPACKET *) pktptr )->checkcode = a[1] + a[2] + a[3] + a[4] + a[5];

 }

