/*
 * MAIN.C - pwICQ's Skin manager entry points
 */

#ifdef __OS2__
 #pragma strings(readonly)
#endif

 #include <string.h>
 #include <stdlib.h>
 #include <pwicqgui.h>

/*---[ Prototipes ]---------------------------------------------------------------------------------------*/

 static void setModeIcon(HICQ, ULONG);
 static void setSysButtonIcon(HICQ);

/*---[ Event processors ]---------------------------------------------------------------------------------*/

 icqDeclareSystemEventProcessor(ICQEVENT_SECONDARY);
 icqDeclareSystemEventProcessor(ICQEVENT_ONLINE);
 icqDeclareSystemEventProcessor(ICQEVENT_OFFLINE);
 icqDeclareSystemEventProcessor(ICQEVENT_HIDE);
 icqDeclareSystemEventProcessor(ICQEVENT_SHOW);
 icqDeclareSystemEventProcessor(ICQEVENT_TIMER);
 icqDeclareSystemEventProcessor(ICQEVENT_TERMINATE);
 icqDeclareSystemEventProcessor(ICQEVENT_CHANGED);
 icqDeclareSystemEventProcessor(ICQEVENT_CONNECTING);
 icqDeclareSystemEventProcessor(ICQEVENT_FINDINGSERVER);
 icqDeclareSystemEventProcessor(ICQEVENT_MESSAGECHANGED);
 icqDeclareUserEventProcessor(ICQEVENT_MESSAGECHANGED);

 icqDeclareGuiEventProcessor(ICQEVENT_TIMER);
 icqDeclareGuiEventProcessor(ICQEVENT_CALLBACK);

/*---[ Event processor table ]----------------------------------------------------------------------------*/

 static const struct icqEventProcessors proctable[] =
 {
    icqGuiEventProcessor(ICQEVENT_TIMER),
    icqSystemEventProcessor(ICQEVENT_ONLINE),
    icqSystemEventProcessor(ICQEVENT_OFFLINE),
    icqSystemEventProcessor(ICQEVENT_SHOW),
    icqSystemEventProcessor(ICQEVENT_HIDE),
    icqSystemEventProcessor(ICQEVENT_SECONDARY),
    icqSystemEventProcessor(ICQEVENT_TIMER),
    icqSystemEventProcessor(ICQEVENT_CHANGED),
    icqSystemEventProcessor(ICQEVENT_CONNECTING),
    icqSystemEventProcessor(ICQEVENT_FINDINGSERVER),
    icqSystemEventProcessor(ICQEVENT_MESSAGECHANGED),
    icqUserEventProcessor(ICQEVENT_MESSAGECHANGED),
    icqGuiEventProcessor(ICQEVENT_CALLBACK),
    icqSystemEventProcessor(ICQEVENT_TERMINATE)
 };

/*---[ Implementation ]-----------------------------------------------------------------------------------*/

 int icqskin_event(HICQ icq, ULONG uin, UCHAR type, USHORT event, ULONG parm)
 {
    int f;
    const struct icqEventProcessors *ptr = proctable;

    icqExecuteGuiListeners(icq, uin, type, event, parm);

    for(f=0;f < ( sizeof(proctable) / sizeof(struct icqEventProcessors) ); f++)
    {
       if(ptr->type == type && ptr->event == event)
          return ptr->exec(icq,uin,parm);
       ptr++;
    }
    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_SECONDARY,icq,uin,parm)
 {
    SKINDATA *cfg              = icqGetPluginDataBlock(icq,module);

	CHKPoint();
	
    icqskin_loadUsersList(icqskin_getDlgItemHandle(cfg->mainWindow,ID_USERLIST), cfg->icq);
	cfg->flags |= SKINDATA_FLAGS_LOADED;

	cfg->eventIcon =
	cfg->readIcon  = ICQICON_INACTIVE;
	
    setSysButtonIcon(icq);
	
    /* Show main window */
    icqskin_setVisible(cfg, TRUE);

    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_TIMER,icq,uin,parm)
 {
    SKINDATA *cfg = icqGetPluginDataBlock(icq,module);

#ifndef __OS2__
	if(!(cfg->flags & SKINDATA_FLAGS_LOADED))
       icqSysEvent_ICQEVENT_SECONDARY(icq,0,0);
#endif       	

    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_ONLINE,icq,uin,parm)
 {
    SKINDATA *cfg = icqGetPluginDataBlock(icq,module);
    cfg->flags &= ~SKINDATA_FLAGS_CONNECTING;

    CHKPoint();
    setModeIcon(icq, icqQueryOnlineMode(icq));

    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_CHANGED,icq,uin,parm)
 {
    SKINDATA *cfg = icqGetPluginDataBlock(icq,module);
    cfg->flags &= ~SKINDATA_FLAGS_CONNECTING;
    setModeIcon(icq, icqQueryOnlineMode(icq));
    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_FINDINGSERVER,icq,uin,parm)
 {
    SKINDATA *skn = icqGetSkinDataBlock(icq);
    icqskin_setFrameIcon(skn->frame, skn, ICQICON_CONNECTING);
    icqskin_setButtonText(icqskin_getDlgItemHandle(skn->mainWindow,ID_MODE),"Finding");
    return 0;
 }
 
 icqBeginSystemEventProcessor(ICQEVENT_CONNECTING,icq,uin,parm)
 {
    SKINDATA *skn = icqGetSkinDataBlock(icq);
    skn->flags |= SKINDATA_FLAGS_CONNECTING;

    CHKPoint();
    icqskin_setFrameIcon(skn->frame, skn, ICQICON_CONNECTING);
    icqskin_setButtonText(icqskin_getDlgItemHandle(skn->mainWindow,ID_MODE),"Trying");

    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_OFFLINE,icq,uin,parm)
 {
    SKINDATA *cfg = icqGetPluginDataBlock(icq,module);
    cfg->flags &= ~SKINDATA_FLAGS_CONNECTING;
	
    setModeIcon(icq, icqQueryOnlineMode(icq));
    CHKPoint();
    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_SHOW,icq,uin,parm)
 {
    CHKPoint();
    icqskin_setVisible(icqGetPluginDataBlock(icq,module), TRUE);
    CHKPoint();
    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_HIDE,icq,uin,parm)
 {
    CHKPoint();
    icqskin_setVisible(icqGetPluginDataBlock(icq,module), FALSE);
    CHKPoint();
    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_TERMINATE,icq,uin,parm)
 {
	icqskin_terminate(icq);
    return 0;
 }

 icqBeginGuiEventProcessor(ICQEVENT_TIMER,icq,uin,parm)
 {
    SKINDATA  *skn = icqGetSkinDataBlock(icq);

	skn->flags ^= SKINDATA_FLAGS_BLINK;
	
    if(skn->flags & SKINDATA_FLAGS_CONNECTING)
       icqskin_setButtonIcon(icqskin_getDlgItemHandle(skn->mainWindow,ID_MODE),ICQICON_ANIMATE+skn->animate);

	if(skn->eventIcon != skn->readIcon)
       icqskin_setButtonIcon(icqskin_getDlgItemHandle(skn->mainWindow,ID_SYSMSG),  (skn->flags & SKINDATA_FLAGS_BLINK) ? skn->eventIcon : skn->readIcon);

    return 0;
 }

 icqBeginSystemEventProcessor(ICQEVENT_MESSAGECHANGED,icq,uin,parm)
 {
    setSysButtonIcon(icq);
    if(icqLoadValue(icq,"TaskICON",1) == 2 || icqLoadValue(icq,"TaskTITLE",1) == 2)
       setModeIcon(icq, icqQueryOnlineMode(icq));
	return 0;
 }

 icqBeginUserEventProcessor(ICQEVENT_MESSAGECHANGED,icq,uin,parm)
 {
    if(icqLoadValue(icq,"TaskICON",1) == 2 || icqLoadValue(icq,"TaskTITLE",1) == 2)
       setModeIcon(icq, icqQueryOnlineMode(icq));
	return 0;
 }

 static void setModeIcon(HICQ icq, ULONG sel)
 {
    const ICQMODETABLE  *mode;
    char			    buffer[0x0100];
    SKINDATA            *skn    = icqGetSkinDataBlock(icq);
    HMSG                msg     = NULL;
    ULONG               uin     = 0;

    CHKPoint();
    icqQueryFirstMessage(icq, &uin, &msg);

    DBGTracex(icqQueryModeTable(icq));

    for(mode = icqQueryModeTable(icq);mode->descr;mode++)
    {
       if(mode->mode == sel)
       {
          DBGMessage("Online mode changed");

          icqskin_setButtonIcon(icqskin_getDlgItemHandle(skn->mainWindow,ID_MODE),mode->icon);
          icqskin_setButtonText(icqskin_getDlgItemHandle(skn->mainWindow,ID_MODE),mode->descr);

          switch(icqskin_queryTaskICON(icq))
          {
          case 1:		// Change icon on mode change
             CHKPoint();
             icqskin_setFrameIcon(skn->frame, skn, mode->icon);
             break;

          case 2:		// Change icon in message
             DBGTracex(msg);
             if(msg && msg->mgr)
                icqskin_setFrameIcon(skn->frame, skn, msg->mgr->icon[0]);
             else
                icqskin_setFrameIcon(skn->frame, skn, mode->icon);
             break;

          }
		
		  CHKPoint();

          switch(icqskin_queryTaskTITLE(icq))
          {
          case 1:		// Change title on mode change
             sprintf(buffer,"%lu - %s",icqQueryUIN(icq),mode->descr);
             DBGMessage(buffer);
             icqskin_setFrameTitle(skn->frame,buffer);
             break;

          case 2:		// Change title in message change
             sprintf(buffer,"%lu - %s",icqQueryUIN(icq),mode->descr);
             DBGMessage(buffer);
             icqskin_setFrameTitle(skn->frame,buffer);
             break;

          }
		
		  CHKPoint();
		
          return;
       }
    }
 }

 icqBeginGuiEventProcessor(ICQEVENT_CALLBACK,icq,uin,parm)
 {
    #pragma pack(1)

    struct _parm
    {
   	   void (* _System callback)(ULONG);
    } *p = (struct _parm *) parm;

    #pragma pack()

    CHKPoint();

    if(p->callback)
       p->callback(parm);

    CHKPoint();

    return 0;	
 }


 static void setSysButtonIcon(HICQ icq)
 {
    SKINDATA  *skn = icqGetSkinDataBlock(icq);
	HMSG      msg  = icqQueryMessage(icq,0);

    if(!msg)
	{
	   skn->eventIcon = skn->readIcon;
	}
	else if(msg->mgr)
	{
       skn->eventIcon = msg->mgr->icon[0];
	   skn->readIcon  = msg->mgr->icon[1];
	}
	else
	{
       skn->eventIcon = ICQICON_UNREADMESSAGE;
	   skn->readIcon  = ICQICON_READMESSAGE;
	}
	
	icqskin_setButtonIcon(icqskin_getDlgItemHandle(skn->mainWindow,ID_SYSMSG),skn->eventIcon);
 }
